#line 1

#define MAX_EFFECTS 4

#define EFFECT_NONE 0
#define EFFECT_FADE 1
#define EFFECT_GHOST 2
#define EFFECT_SPLASH 3
#define EFFECT_CHROME 4
#define EFFECT_BANG 5
#define EFFECT_ELECTRO 6
#define EFFECT_TURBO 7
#define EFFECT_GLOW 8
#define EFFECT_FOX 9
#define EFFECT_WATERBOX 10
#define EFFECT_SHOCKWAVE 11
#define EFFECT_BOMB 12

#ifdef MODEL_EFFECTS
// model fx
uniform int modelEffect;
uniform vec2 modelfxParams;
uniform vec3 modelLight;
#ifndef USE_ENV
// env vars
uniform vec3 modelEnvMatX;
uniform vec3 modelEnvMatY;
#endif  // USE_ENV
#endif  // MODEL_EFFECTS

#ifdef MESH_EFFECTS
// mesh fx
uniform int numEffects;
uniform int meshEffect[MAX_EFFECTS];
uniform vec3 meshfxParams[MAX_EFFECTS];
uniform vec3 meshfxParams2[MAX_EFFECTS];
uniform float wboxSineTable[9];

int Mod(float x, float y)
{
  float r = mod(x, y);
  return int(clamp(r, 0.0, y-1.0));
}

void ProcessMeshEffect(inout vec4 pos, int meshEffect, vec3 meshfxParams, vec3 meshfxParams2)
{
  // waterbox mesh fx
  if (meshEffect == EFFECT_WATERBOX) {
    vec3 bboxMin = meshfxParams.xyz;
    vec3 bboxMax = meshfxParams2.xyz;
    if (all(greaterThan(inPosition.xyz, bboxMin)) && 
        all(lessThan(inPosition.xyz, bboxMax))) {
      float hash = floor(abs(inPosition.x * inPosition.z));
      vec3 delta = vec3(
          wboxSineTable[Mod(hash+0.0, 9.0)], 
          wboxSineTable[Mod(hash+3.0, 9.0)] + 6.0, 
          wboxSineTable[Mod(hash+6.0, 9.0)]);
#ifdef MODEL_EFFECTS
      delta *= 0.2;
#endif  // MODEL_EFFECTS
      pos.xyz += delta;
    }

  // shockwave mesh fx
  } else if (meshEffect == EFFECT_SHOCKWAVE) {
    vec3 objPos = meshfxParams.xyz;
    float reach = meshfxParams2.x;

    vec3 delta = pos.xyz - objPos;
    float dist = dot(delta, delta);
    if (dist < reach * reach) {
      dist = sqrt(dist);
      float pull = (reach - dist) * 0.1;
      pull = min(pull, dist * 0.5);
      pos.xyz -= delta * (pull / dist);
    }

  // putty bomb mesh fx
  } else if (meshEffect == EFFECT_BOMB) {
    vec3 objPos = meshfxParams.xyz;
    float rad = meshfxParams2.x;
    float timeStep = meshfxParams2.y;

    vec3 delta = pos.xyz - objPos;
    float dist = dot(delta, delta);
    dist = sqrt(dist);
    float scale = (0.5 - timeStep) / 0.5;
    float push = (64.0 - abs(rad - dist)) * scale;
    pos.xyz += delta * (max(push, 0.0) / dist);
  }
}
#endif  // MESH_EFFECTS

#ifdef MODEL_EFFECTS
void ProcessWeaponEffect(inout vec4 pos)
{
  float timeStep = modelfxParams.x;
  float time = inTexCoord.s;
  float add = inTexCoord.t;
  time += add * timeStep;

  // bomb bang
  if (modelEffect == EFFECT_BANG) {
    varTexCoord.s = sin(time) * (12.0 / 256.0) + (36.0 / 256.0);
    varTexCoord.t = cos(time) * (12.0 / 256.0) + (68.0 / 256.0);

  // electro pulse
  } else if (modelEffect == EFFECT_ELECTRO) {
    varTexCoord.s = sin(time) * (24.0 / 256.0) + (96.0 / 256.0);
    varTexCoord.t = cos(time) * (24.0 / 256.0) + (96.0 / 256.0);

  // turbo
  } else if (modelEffect == EFFECT_TURBO) {
    varTexCoord.s = sin(time) * (10.0 / 256.0) + (96.0 / 256.0);
    varTexCoord.t = cos(time) * (10.0 / 256.0) + (160.0 / 256.0);

  // bomb glow
  } else if (modelEffect == EFFECT_GLOW) {
    varTexCoord.s = sin(time) * (12.0 / 256.0) + (36.0 / 256.0);
    varTexCoord.t = cos(time) * (12.0 / 256.0) + (44.0 / 256.0);

  // fox
  } else if (modelEffect == EFFECT_FOX) {
    varTexCoord.s = sin(time) * (24.0 / 256.0) + (160.0 / 256.0);
    varTexCoord.t = cos(time) * (24.0 / 256.0) + (160.0 / 256.0);
  }
}

void ProcessModelEffect(inout vec4 pos)
{
  // fade alpha
  if (modelEffect == EFFECT_FADE) {
    float timeStep = modelfxParams.x;
    varColor.a = 1.0 - timeStep;

  // ghost car
  } else if (modelEffect == EFFECT_GHOST) {
    float ghostPos = modelfxParams.x;
    float ghostMul = modelfxParams.y;
    varColor.a = abs((pos.z + ghostPos) * ghostMul);
    varColor.a = 1.0 - varColor.a;

  // water splash
  } else if (modelEffect == EFFECT_SPLASH) {
    float timeStep = modelfxParams.x;
    float frame = inPosition.w * timeStep;
    if (frame < 16.0) {
      float grav = 384.0 * timeStep;
      vec3 vel = inNormal;
      vel.y += grav;
      pos.xyz += vel * timeStep;
      pos.w = 1.0;

      float rgb = (16.0 - frame) / 32.0;
      varColor.rgb = vec3(rgb);
      varTexCoord.s += floor(frame) / 16.0;
    }

  // chrome ball
  } else if (modelEffect == EFFECT_CHROME) {
    varTexCoord.s = dot(inNormal, modelEnvMatX) + 0.5;
    varTexCoord.t = dot(inNormal, modelEnvMatY) + 0.6;

  // weapon fx
  } else {
    ProcessWeaponEffect(pos);
  }
}
#endif  // MODEL_EFFECTS

void ProcessEffects(inout vec4 pos)
{
#ifdef MODEL_EFFECTS
  if (modelEffect != EFFECT_NONE) {
    ProcessModelEffect(pos);
  }
#endif  // MODEL_EFFECTS

#ifdef MESH_EFFECTS
  if (numEffects > 0) {
    ProcessMeshEffect(pos, meshEffect[0], meshfxParams[0], meshfxParams2[0]);
  }
  if (numEffects > 1) {
    ProcessMeshEffect(pos, meshEffect[1], meshfxParams[1], meshfxParams2[1]);
  }
  if (numEffects > 2) {
    ProcessMeshEffect(pos, meshEffect[2], meshfxParams[2], meshfxParams2[2]);
  }
  if (numEffects > 3) {
    ProcessMeshEffect(pos, meshEffect[3], meshfxParams[3], meshfxParams2[3]);
  }
#endif  // MESH_EFFECTS

#ifdef MODEL_EFFECTS
  // add any tinting
  varColor.rgb += modelLight;
#endif  // MODEL_EFFECTS
}
